bl_info = {
    "name": "Make Cookie-Cutter",
    "category": "Object",
    "description": "Transform your 2D-Mesh into a cookie-cutter.",
    "author": "Oromis - Valentin S.",
    "version": (1, 1),
    "blender": (2, 79, 0),
    "location": "Space Search Only 'Make Cookie-Cutter'",
}

import bpy
import math


class MakeCookieCutter(bpy.types.Operator):
    """Make Cookie-Cutter"""
    bl_idname = "object.make_cookie_cutter"
    bl_label = "Make Cookie-Cutter"
    bl_options = {'REGISTER', 'UNDO'}
    # moved assignment from execute() to the body of the class...
    CurveResolutionProp = bpy.props.IntProperty(name="Resolution", default=8, min=1, max=64, description="Decrease Curve-Resolution, useless for Mesh-Objects")
    DecimateValueProp = bpy.props.IntProperty(name="Decimate", default=0, min=0, max=180, description="Higher values might solve some issues")
    CutterHeightProp = bpy.props.FloatProperty(name="Base Height", default=0.17, min=0, max=100)
    CutterBrimProp = bpy.props.FloatProperty(name="Brim Height", default=0.06, min=0, max=100)
    CutterEdgeHeightProp = bpy.props.FloatProperty(name="Edge Height", default=0.04, min=0, max=100)
    TotalWidthProp = bpy.props.FloatProperty(name="Brim Width", default=0.13, min=0, max=100)
    InsetAProp = bpy.props.FloatProperty(name="Base Width", default=0.04, min=0, max=100)
    MultiplyProp = bpy.props.FloatProperty(name="MultiplyProp", default=1, min=0, max=100)
    CorrectAspectProp = bpy.props.BoolProperty(name="Mirror Correction")


    def execute(self, context):

        
# Start of Script      

# --> Used Variables

#		Defined by user:
#		TotalWidth:						Total width of brim
#		CutterHeight:					Height of cutter without brim + cutting edge; thicker dough needs higher values
#		CutterBrim:						Height of brim, higher values increase stability
#		CutterEdgeHeight:			Height of cutting edge above counter edge
#		InsetA:								First Inset Value, defines width of body
#		InsetB:								Second Inset Value, define angle of body -> keep as small as possible

#		Defined by script:
#		VertexCount:					Count of vertices on the inner edge. Used for controlling extrusion parts
#		ZScale:								Half of sum of CutterHeights

        scene = bpy.context.scene

        # Variable declaration
        CurveResolution = self.CurveResolutionProp
        DecimateValue = math.radians(self.DecimateValueProp)
        TotalWidth = -self.TotalWidthProp
        CutterHeight = self.CutterHeightProp * self.MultiplyProp
        CutterBrim = self.CutterBrimProp * self.MultiplyProp
        CutterEdgeHeight = self.CutterEdgeHeightProp * self.MultiplyProp
        InsetA = self.InsetAProp * self.MultiplyProp
        InsetB = 0.001
        CorrectAspect = self.CorrectAspectProp
        ZScale = ((3*CutterHeight + 3*CutterBrim + CutterEdgeHeight) / -3) + 2/3*CutterHeight + 1/6*CutterEdgeHeight

        # !!!!! --- RESET ORIGIN AND MODEL LOCATION --- !!!!! #
        for obj in bpy.context.selected_objects:
            if obj.type == 'CURVE':
                bpy.context.object.data.dimensions = '2D'
                bpy.context.object.data.fill_mode = 'NONE'
                bpy.context.object.data.resolution_u = CurveResolution
                bpy.ops.object.convert(target='MESH')
            if obj.type == 'MESH':
                bpy.ops.object.mode_set(mode='OBJECT')									# Switch do object mode
                bpy.ops.object.origin_set(type='ORIGIN_GEOMETRY')				# Set origin to geometry
                obj.scale.z = 0.0																				# Reset Z-scale to zero
                obj.location.x = 0.0																		# Reset X-location to zero
                obj.location.y = 0.0																		# Reset Y-location to zero
                obj.location.z = 0.0																		# Reset Z-location to zero
                
                bpy.ops.object.transform_apply(location=True, rotation=True, scale=True)
                

        ##!!!!! SET ACTIVE OBJECT INTO EDIT MODE AND REMOVE DOUBLES
                bpy.ops.object.mode_set(mode='EDIT')                    # Switch do Edit mode
                bpy.ops.mesh.select_mode(type="VERT")										# Set selection mode to vertex
                bpy.ops.mesh.select_all(action='SELECT')                # Select all vertices
                bpy.ops.mesh.remove_doubles(threshold=0.0001)           # Remove doubles (just in case)



        ##!!!!! EXTRUDE AND MOVE ON Z
        from mathutils import Vector
        ExtrudeVector = Vector((0.0,0.0,1.0))														# Define direction of extrusion

        bpy.ops.mesh.extrude_region_move(TRANSFORM_OT_translate={"value":ExtrudeVector})

        bpy.ops.mesh.select_all(action='SELECT')              				  # Select all vertices
        bpy.ops.mesh.normals_make_consistent(inside=False)    	        # Recalculate normals
        
        #Add, configure and apply Decimate-Modifier
        bpy.ops.object.mode_set(mode='OBJECT')									# Switch do object mode
        bpy.ops.object.modifier_add(type='DECIMATE')
        bpy.context.object.modifiers["Decimate"].decimate_type = 'DISSOLVE'
        bpy.context.object.modifiers["Decimate"].angle_limit = DecimateValue
        bpy.ops.object.modifier_apply(apply_as='DATA', modifier="Decimate")
        bpy.ops.object.mode_set(mode='EDIT')									# Switch do edit mode
                
        VertexCount = int(len(obj.data.vertices)/2) 										# Count current vertices



        # Extrude and move on individual Edges
        bpy.ops.mesh.extrude_region_shrink_fatten(MESH_OT_extrude_region={"mirror":False}, TRANSFORM_OT_shrink_fatten={"value": TotalWidth, "use_even_offset":False, "mirror":False, "proportional":'DISABLED', "proportional_edit_falloff":'SMOOTH', "proportional_size":1, "snap":False, "snap_target":'CLOSEST', "snap_point":(0, 0, 0), "snap_align":False, "snap_normal":(0, 0, 0), "release_confirm":False, "use_accurate":False})


        ##!!!!! Delete all vertices above Z
        bpy.ops.mesh.select_all(action='DESELECT')											# Deselect all vertices
        vert = bpy.context.object.data.vertices													# Vert == Vertex count
        bpy.ops.object.mode_set(mode='OBJECT')													# Switch to object mode

        for v in vert:																									# For all vertices...
            if v.co[2] > 0:																							# ...check if vertice is above Z=0
                v.select = True																					# If true: Select vertice

        bpy.ops.object.mode_set(mode = 'EDIT')													# Switch to edit mode
        bpy.ops.mesh.delete(type='VERT')																# Delete selected vertices
        bpy.ops.object.mode_set(mode = 'OBJECT')												# Switch to object mode


        ##!!!!! INSET FACES TO PREPARE EXTRUSION
        for obj in bpy.context.selected_objects:
          if obj.type == 'MESH':
            bpy.ops.object.mode_set(mode='EDIT')												# Switch do edit mode
            bpy.ops.mesh.select_all(action='SELECT')										# Select all vertices

            bpy.ops.mesh.inset(thickness=InsetA, depth=0)								# Inset faces
            bpy.ops.mesh.inset(thickness=InsetB, depth=0)								# Inset faces

        ##!!!!! SELECT AND EXTRUDE CUTTERBRIM
            bpy.ops.mesh.select_all(action='SELECT')										# Select all vertices
            bpy.ops.transform.translate(value=(0, 0, CutterBrim), constraint_axis=(False, False, True), constraint_orientation='GLOBAL', mirror=False, proportional='DISABLED', proportional_edit_falloff='SMOOTH', proportional_size=1)

            bpy.ops.mesh.select_all(action='DESELECT')									# Deselect all faces

        ##!!!!! SELECT AND EXTRUDE CUTTERHEIGHT
        		# Load saved vertices
            bpy.ops.object.mode_set(mode = 'EDIT')											# Switch to edit mode
            bpy.ops.mesh.select_all(action = 'DESELECT')								# Deselect all faces
            bpy.ops.object.mode_set(mode = 'OBJECT')										# Switch to object mode
            for y in range(VertexCount):																# For all (previously counted) vertices...
              obj.data.vertices[y].select = True												# ...select vertex
            bpy.ops.object.mode_set(mode = 'EDIT') 											# Switch to edit mode

            bpy.ops.mesh.select_more()
            bpy.ops.transform.translate(value=(0, 0, CutterHeight), constraint_axis=(False, False, True), constraint_orientation='GLOBAL', mirror=False, proportional='DISABLED', proportional_edit_falloff='SMOOTH', proportional_size=1)

            bpy.ops.mesh.select_all(action='DESELECT')									# Deselect all faces

        ##!!!!! SELECT AND EXTRUDE CUTTERBRIM
        		# Load saved vertices
            bpy.ops.object.mode_set(mode = 'EDIT')											# Switch to edit mode
            bpy.ops.mesh.select_all(action = 'DESELECT')								# Deselect all faces
            bpy.ops.object.mode_set(mode = 'OBJECT')										# Switch to object mode
            for y in range(VertexCount):																# For all (previously counted) vertices...
              obj.data.vertices[y].select = True												# ...select vertex
            bpy.ops.object.mode_set(mode = 'EDIT') 											# Switch to edit mode
                
            bpy.ops.transform.translate(value=(0, 0, CutterEdgeHeight), constraint_axis=(False, False, True), constraint_orientation='GLOBAL', mirror=False, proportional='DISABLED', proportional_edit_falloff='SMOOTH', proportional_size=1)

                

        ##!!!!! EXTRUDE AND MOVE DOWN ON Z <<<<<
            bpy.ops.mesh.select_all(action='SELECT')										# Select all faces
            from mathutils import Vector
            ExtrudeVector = Vector((0.0,0.0,ZScale))
            bpy.ops.mesh.extrude_region_move(TRANSFORM_OT_translate={"value":ExtrudeVector})


        ##!!!!! SCALE EXTRUDED VERTICES TO Z-0 <<<<<
            bpy.ops.object.mode_set(mode='EDIT')												# Switch to edit mode
            bpy.ops.transform.resize(																		# Resize Z -> Zero
              value=(1, 1, 0.0), 
              constraint_axis=(False, False, False), 
              constraint_orientation='GLOBAL', 
              mirror=False, 
              proportional='DISABLED', 
              proportional_edit_falloff='SMOOTH', 
              proportional_size=1)


       				# >>>>> RECALCULATE NORMALS <<<<<
            bpy.ops.mesh.select_all(action='SELECT') 		              # Select all faces
            if CorrectAspect == True:	
                bpy.ops.transform.resize(value=(-1, 1, 1), constraint_axis=(True, False, False), constraint_orientation='GLOBAL', mirror=False, proportional='DISABLED', proportional_edit_falloff='SMOOTH', proportional_size=1)
            bpy.ops.mesh.normals_make_consistent(inside=False)    	  # Recalculate normals

            bpy.ops.mesh.select_all(action='DESELECT') 		              # Deselect all
            bpy.ops.object.mode_set(mode='OBJECT')                      # Switch to object mode)
            
            for y in range(VertexCount):																# For all (previously counted) vertices...
              obj.data.vertices[VertexCount*3 + y].select = True												# ...select vertex
            for y in range(VertexCount):																# For all (previously counted) vertices...
              obj.data.vertices[VertexCount*8 + y].select = True												# ...select vertex
            for y in range(VertexCount):																# For all (previously counted) vertices...
              obj.data.vertices[VertexCount*9 + y].select = True												# ...select vertex
            for y in range(VertexCount):																# For all (previously counted) vertices...
              obj.data.vertices[VertexCount*10 + y].select = True												# ...select vertex
            for y in range(VertexCount):																# For all (previously counted) vertices...
              obj.data.vertices[VertexCount*11 + y].select = True												# ...select vertex
            for y in range(VertexCount):																# For all (previously counted) vertices...
              obj.data.vertices[VertexCount*5 + y].select = True												# ...select vertex
            bpy.ops.object.mode_set(mode = 'EDIT') 											# Switch to edit mode
            bpy.ops.mesh.dissolve_verts()
            
            bpy.ops.mesh.select_all(action='DESELECT') 		              # Select all
            bpy.ops.object.mode_set(mode = 'OBJECT') 								   	# Switch to edit mode

        # End of Script

        return {'FINISHED'}

def register():
    bpy.utils.register_class(MakeCookieCutter)
    


def unregister():
    bpy.utils.unregister_class(MakeCookieCutter)


if __name__ == "__main__":
    register()